@@ -1,44 +1,60 @@
 #!/usr/bin/env python
 # coding=utf-8
+from __future__ import print_function
 from .rpc_client import rpc_client
 from . import __version__
-from urlparse import urlparse
 import feedparser
 import argparse
 import hashlib
-import urllib
 import base64
 import redis
-import types
 import json
 import yaml
 import time
 import sys
 import os
 
+if sys.version_info[0] < 3:
+    from urlparse import urlparse
+    from urllib import unquote
+    string_types = unicode
+else:
+    from urllib.parse import urlparse
+    from urllib.parse import unquote
+    string_types = str
+
+
 CONF_FILE = '~/.trmaid.yaml'
 CONF_FILE_FULL = os.path.expanduser(CONF_FILE)
 
 
 def msg_and_exit(*msg):
-    print '\n'.join(map(str, msg))
+    print('\n'.join(map(str, msg)))
     sys.exit(1)
 
 
+def want_bytes(elem, encoding='utf-8'):
+    if sys.version_info[0] < 3:
+        return elem
+    elif isinstance(elem, string_types):
+        elem = elem.encode(encoding)
+    return elem
+
+
 def utfme(elem):
-    if isinstance(elem, types.DictType):
+    if isinstance(elem, dict):
         for key in elem.keys():
             elem[key] = utfme(elem[key])
-    elif isinstance(elem, (types.ListType, types.TupleType)):
+    elif isinstance(elem, (list, tuple)):
         res = []
         for e in elem:
             res.append(utfme(e))
-        if isinstance(elem, types.TupleType):
+        if isinstance(elem, tuple):
             elem = tuple(res)
         else:
             elem = res
-    elif isinstance(elem, basestring):
-        elem = elem.encode('utf-8')
+    elif isinstance(elem, string_types):
+        elem = want_bytes(elem)
     return elem
 
 
@@ -74,36 +90,35 @@ class TRMaid(object):
             self.rds = redis.from_url(rds_url)
             self.rds.ping()
         except redis.exceptions.ConnectionError:
-            msg_and_exit("Please install and run redis on %s" % rds_url)
+            msg_and_exit('Please install and run redis on %s' % rds_url)
 
         # transmission conf check
         if 'url' not in transmission:
             msg_and_exit('transmission.url not configured')
-        elif 'username' not in transmission:
-            msg_and_exit('transmission.username not configured')
-        elif 'password' not in transmission:
-            msg_and_exit('transmission.password not configured')
 
         # rpc stuff
         verify = general.get('verify', True)
 
         self.rpc = rpc_client(
-            transmission['url'],
-            transmission['username'],
-            transmission['password'],
-            verify=verify
+            url=transmission['url'],
+            username=transmission.get('username'),
+            password=transmission.get('password'),
+            verify=verify,
         )
 
     def msg(self, *msgs):
         if self.verbose:
-            print ' => ' + '\n    '.join(msgs)
+            print(' => ' + '\n    '.join(msgs))
 
     def verbose_func(self, func, *args):
         if self.verbose:
-            print '%s(%s)' % (func.__name__, json.dumps(args, indent=2)),
+            print ('%s(%s)' % (
+                func.__name__,
+                ', '.join([json.dumps(x, indent=2) for x in args]),
+            ), end='')
         result = func(*args)
         if self.verbose:
-            print '=>', json.dumps(result, indent=2)
+            print('=>', json.dumps(result, indent=2))
         return result
 
     def handle_session(self):
@@ -190,7 +205,7 @@ class TRMaid(object):
                 self.rds.expire(
                     'trmaid:blocklist-update', blocklist_update_interval * 3600
                 )
-                # self.verbose_func(self.rpc.blocklist_update)
+                self.verbose_func(self.rpc.blocklist_update)
 
         if req:
             self.verbose_func(self.rpc.session_set, req)
@@ -364,10 +379,14 @@ class TRMaid(object):
 
         return req
 
+    @staticmethod
+    def md5(text):
+        return hashlib.md5(text.encode('utf-8')).hexdigest()
+
     def add_links(self, section, subsection, links, rss=False):
         if rss:
             rdskey = 'trmaid:added:%s:%s' % (
-                hashlib.md5(subsection).hexdigest(),
+                self.md5(subsection),
                 urlparse(subsection).hostname,
             )
             existing = self.rds.lrange(rdskey, 0, -1)
@@ -377,9 +396,8 @@ class TRMaid(object):
 
         ids = []
         add_req = self.get_torrent_add_args(section, subsection)
-
         for link in links:
-            xhash = hashlib.md5(link).hexdigest()
+            xhash = want_bytes(self.md5(link))
             if xhash in existing:
                 continue
             add_req['filename'] = link
@@ -397,6 +415,7 @@ class TRMaid(object):
             if arguments.get('torrent-added', None):
                 ids.append(arguments['torrent-added']['id'])
 
+
         if ids:
             if rdskey:
                 self.rds.ltrim(rdskey, 0, 1000)
@@ -422,7 +441,7 @@ class TRMaid(object):
             for entry in feedparser.parse(feed_url).entries:
                 if not entry.link:
                     continue
-                yield urllib.unquote(entry.link)
+                yield unquote(entry.link)
 
         for feed_url in self.conf.get('rss', {}).keys():
             self.msg('handling feed %s...' % feed_url[:40])
@@ -441,7 +460,7 @@ class TRMaid(object):
         folders = self.conf.get('watch-folders')
         if not folders:
             return
-        for folder, conf in folders.iteritems():
+        for folder in folders.keys():
             self.msg('handling folder %s' % folder)
             ids = []
             add_req = self.get_torrent_add_args('watch-folders', folder)
@@ -503,7 +522,7 @@ def main():
     args = parser.parse_args()
 
     if args.version:
-        print 'trmaid %s' % __version__
+        print('trmaid %s' % __version__)
         return
 
     TRMaid(
